use crossterm::event::{KeyCode, KeyEvent, KeyEventKind, KeyModifiers};
// use minifb::{InputCallback, Key};

use crate::memory::{MemHandle, MemoryWriter};

pub struct Keyboard {
    memory: MemHandle,
}

impl Keyboard {
    pub fn new(memory: MemHandle) -> Self {
        Self { memory }
    }
    pub fn read_keys(&self, key: KeyEvent) {
        let mut row0 = 0;
        let mut row1 = 0;
        let mut row2 = 0;
        let mut row3 = 0;
        let mut row4 = 0;
        let mut row5 = 0;
        if key.kind == KeyEventKind::Press || key.kind == KeyEventKind::Repeat {
            match key.modifiers {
                KeyModifiers::SHIFT => row2 ^= 0b1000_0000,
                KeyModifiers::CONTROL => row3 ^= 0b1000_0000,
                KeyModifiers::ALT => row4 ^= 0b1000_0000,
                KeyModifiers::META => row5 ^= 0b1000_0000,
                KeyModifiers::SUPER => row5 ^= 0b0010_0000,
                _ => {}
            };

            match key.code {
                KeyCode::Esc => row0 ^= 0b1000_0000,
                KeyCode::Char('w') => row0 ^= 0b0100_0000,
                KeyCode::Char('e') => row0 ^= 0b0010_0000,
                KeyCode::Char('r') => row0 ^= 0b0001_0000,
                KeyCode::Char('t') => row0 ^= 0b0000_1000,
                KeyCode::Char('u') => row0 ^= 0b0000_0100,
                KeyCode::Char('o') => row0 ^= 0b0000_0010,
                KeyCode::Backspace => row0 ^= 0b0000_0001,
                KeyCode::Tab => row1 ^= 0b1000_0000,
                KeyCode::Char('q') => row1 ^= 0b0100_0000,
                KeyCode::Char('s') => row1 ^= 0b0010_0000,
                KeyCode::Char('g') => row1 ^= 0b0001_0000,
                KeyCode::Char('y') => row1 ^= 0b0000_1000,
                KeyCode::Char('i') => row1 ^= 0b0000_0100,
                KeyCode::Char('p') => row1 ^= 0b0000_0010,
                KeyCode::Enter => row1 ^= 0b0000_0001,
                KeyCode::Char('d') => row2 ^= 0b0100_0000,
                KeyCode::Char('v') => row2 ^= 0b0010_0000,
                KeyCode::Char('h') => row2 ^= 0b0001_0000,
                KeyCode::Char('k') => row2 ^= 0b0000_1000,
                KeyCode::Char('\'') => row2 ^= 0b0000_0100,
                KeyCode::Char('/') => row2 ^= 0b0000_0010,
                KeyCode::Char('a') => row2 ^= 0b0000_0001,
                KeyCode::Char('z') => row3 ^= 0b0100_0000,
                KeyCode::Char('f') => row3 ^= 0b0010_0000,
                KeyCode::Char('b') => row3 ^= 0b0001_0000,
                KeyCode::Char('j') => row3 ^= 0b0000_1000,
                KeyCode::Char('l') => row3 ^= 0b0000_0100,
                KeyCode::Char('2') => row3 ^= 0b0000_0010,
                KeyCode::Char('4') => row3 ^= 0b0000_0001,
                KeyCode::Char('x') => row4 ^= 0b0100_0000,
                KeyCode::Char('c') => row4 ^= 0b0010_0000,
                KeyCode::Char('n') => row4 ^= 0b0001_0000,
                KeyCode::Char('m') => row4 ^= 0b0000_1000,
                KeyCode::Char(',') => row4 ^= 0b0000_0100,
                KeyCode::Char('1') => row4 ^= 0b0000_0010,
                KeyCode::Char('3') => row4 ^= 0b0000_0001,
                KeyCode::Char(' ') => row5 ^= 0b0100_0000,
                _ => {}
            }
        };

        self.memory.write(0x4400, row0);
        self.memory.write(0x4401, row1);
        self.memory.write(0x4402, row2);
        self.memory.write(0x4403, row3);
        self.memory.write(0x4404, row4);
        self.memory.write(0x4405, row5);
    }
}

impl MemoryWriter for Keyboard {
    fn write(&self, address: u16, data: u8) {
        self.memory.write(address, data);
    }
}

// impl InputCallback for Keyboard {
//     fn add_char(&mut self, _uni_char: u32) {}
//     fn set_key_state(&mut self, key: Key, _state: bool) {
//         let mut row0 = 0;
//         let mut row1 = 0;
//         let mut row2 = 0;
//         let mut row3 = 0;
//         let mut row4 = 0;
//         let mut row5 = 0;

//         match key {
//             Key::Escape => row0 ^= 0b1000_0000,
//             Key::W => row0 ^= 0b0100_0000,
//             Key::E => row0 ^= 0b0010_0000,
//             Key::R => row0 ^= 0b0001_0000,
//             Key::T => row0 ^= 0b0000_1000,
//             Key::U => row0 ^= 0b0000_0100,
//             Key::O => row0 ^= 0b0000_0010,
//             Key::Backspace => row0 ^= 0b0000_0001,
//             Key::Tab => row1 ^= 0b1000_0000,
//             Key::Q => row1 ^= 0b0100_0000,
//             Key::S => row1 ^= 0b0010_0000,
//             Key::G => row1 ^= 0b0001_0000,
//             Key::Y => row1 ^= 0b0000_1000,
//             Key::I => row1 ^= 0b0000_0100,
//             Key::P => row1 ^= 0b0000_0010,
//             Key::Enter => row1 ^= 0b0000_0001,
//             Key::LeftShift | Key::RightShift => row2 ^= 0b1000_0000,
//             Key::D => row2 ^= 0b0100_0000,
//             Key::V => row2 ^= 0b0010_0000,
//             Key::H => row2 ^= 0b0001_0000,
//             Key::K => row2 ^= 0b0000_1000,
//             Key::Apostrophe => row2 ^= 0b0000_0100,
//             Key::Slash => row2 ^= 0b0000_0010,
//             Key::A => row2 ^= 0b0000_0001,
//             Key::LeftCtrl | Key::RightCtrl => row3 ^= 0b1000_0000,
//             Key::Z => row3 ^= 0b0100_0000,
//             Key::F => row3 ^= 0b0010_0000,
//             Key::B => row3 ^= 0b0001_0000,
//             Key::J => row3 ^= 0b0000_1000,
//             Key::L => row3 ^= 0b0000_0100,
//             Key::Key2 => row3 ^= 0b0000_0010,
//             Key::Key4 => row3 ^= 0b0000_0001,
//             Key::LeftAlt | Key::RightAlt => row4 ^= 0b1000_0000,
//             Key::X => row4 ^= 0b0100_0000,
//             Key::C => row4 ^= 0b0010_0000,
//             Key::N => row4 ^= 0b0001_0000,
//             Key::M => row4 ^= 0b0000_1000,
//             Key::Comma => row4 ^= 0b0000_0100,
//             Key::Key1 => row4 ^= 0b0000_0010,
//             Key::Key3 => row4 ^= 0b0000_0001,
//             Key::LeftSuper => row5 ^= 0b1000_0000,
//             Key::Space => row5 ^= 0b0100_0000,
//             Key::RightSuper => row5 ^= 0b0010_0000,
//             _ => {}
//         };

//         self.memory.write(0x4400, row0);
//         self.memory.write(0x4401, row1);
//         self.memory.write(0x4402, row2);
//         self.memory.write(0x4403, row3);
//         self.memory.write(0x4404, row4);
//         self.memory.write(0x4405, row5);
//     }
// }
